"""Obsidian MCP - Model Context Protocol server for Obsidian vaults.

A FastMCP server that enables AI assistants to interact with Obsidian vaults
through direct filesystem access. Features include note CRUD operations,
advanced search with persistent indexing, tag management, and link analysis.
"""

__version__ = "2.0.0"
__author__ = "Nate Strong"
__email__ = "nathanstrong.fx@gmail.com"

__all__ = ["__version__"]